//*****************************************************************
// 
//  JAVA Source for com.idautomation.datamatrix; 2017
//  
//	 Copyright IDAutomation.com, Inc. All rights reserved.
//	Redistribution and use of this code in source and/or binary 
//	forms, with or without modification, are permitted provided 
//	that: (1) all copies of the source code retain the above 
//	unmodified copyright notice and this entire unmodified 
//	section of text, (2) You or Your organization owns a valid 
//	Developer License to this product from IDAutomation.com 
//	and, (3) when any portion of this code is bundled in any 
//	form with an application, a valid notice must be provided 
//	within the user documentation, start-up screen or in the 
//	help-about section of the application that specifies 
//	IDAutomation.com as the provider of the Software bundled 
//	with the application.
//*****************************************************************

package com.idautomation.datamatrix.encoder;

import javax.imageio.ImageIO;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;





/** This is the class that actually saves the image of the DataMatrix class.*/
public class barCodeEncoder {

   String sFile;
   String sFormat;
   com.idautomation.datamatrix.DataMatrix bc;
   public boolean result;
/** Constructor used to save an image of the current state of the DataMatrix object.
@param c DataMatrix object.
@param psFormat format to save the image as a "GIF" or "JPEG"
@param psFile Name of the saved image file, "Test.gif"
@see encodeImage(com.idautomation.datamatrix.DataMatrix c,String psFormat,String psFile)*/
  public barCodeEncoder(com.idautomation.datamatrix.DataMatrix c,String psFormat,String psFile) {

     sFormat=psFormat;
     sFile=psFile;
     bc=c;
     result=encode();
  }

/** Has the same functionality as the constructor, can be used to encode images without the need to reinstantiate the barCodeEncoder object.
@param c DataMatrix object.
@param psFormat format to save the image as a "GIF" or "JPEG"
@param psFile Name of the saved image file, "Test.gif"
@see barCodeEncoder(com.idautomation.datamatrix.DataMatrix c,String psFormat,String psFile)*/
  public void encodeImage(com.idautomation.datamatrix.DataMatrix c,String psFormat,String psFile) 
  {
     sFormat=psFormat;
     sFile=psFile;
     bc=c;
     result=encode();
  }
  
   //Luigi implemented EPS and SVG image options (BRJ added to Data Matrix 1/18/2017)
   private boolean encode() {
        if (sFormat.toUpperCase().compareTo("GIF") == 0) {
            return saveToGIF();
        } else if (sFormat.toUpperCase().compareTo("JPEG") == 0) {
            return saveToJPEG();
        } else if (sFormat.toUpperCase().compareTo("EPS") == 0) {
            return saveToEPS();
        } else if (sFormat.toUpperCase().compareTo("SVG") == 0) {
            return saveToSVG();
        }
        return false;
    }

   private boolean saveToGIF() 
   {
      String v=java.lang.System.getProperty("java.version");
      if (v.indexOf("1.1")==0) 
          return false;
       try 
       {
            // *** This method was added to auto size the images the first time ***
            if (bc.autoSize) 
            {
                //TB, 10/2/2009, fix bug #363, must set a temporary size, otherwise a null image is created.
                bc.setSize(170,90);
                java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( 100,100,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                //java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                java.awt.Graphics imgTempGraphics = imageTemp.createGraphics();
                bc.paint(imgTempGraphics);
                bc.invalidate();
                imgTempGraphics.dispose();
            }

            //create bufferred image
            java.awt.image.BufferedImage image = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
            //java.awt.Image image=c.createImage(c.getSize().width,c.getSize().height);
            java.awt.Graphics imgGraphics = image.createGraphics();

            bc.paint(imgGraphics);

            // open file
            java.io.File f=new java.io.File(sFile);
            f.delete();
            java.io.FileOutputStream of=new java.io.FileOutputStream(f);

            // encode buffered image to a gif
            GifEncoder encoder = new GifEncoder(image,of);
            encoder.encode();
            of.close();
        }
        catch (Exception e) 
        {
            return false;
        }

        return true;
    }

	
	
		   //Luigi implemented saveToSVG(BRJ added to Data Matrix 1/17/2016)
	   private boolean saveToSVG() {
        try {
            String svgOutput = bc.createSVG();

            BufferedWriter writer;
            writer = new BufferedWriter(new FileWriter(sFile));
            writer.write(svgOutput);
            writer.close( );

        } catch (IOException e) {
            return false;
        }

        return true;
    }
	
	//Luigi implemented saveToEPS(BRJ added to Data Matrix 1/17/2016)
	private boolean saveToEPS() {
        try {
            String epsOutput = bc.createEPS();

            BufferedWriter writer;
            writer = new BufferedWriter(new FileWriter(sFile));
            writer.write(epsOutput);
            writer.close( );

        } catch (IOException e) {
            return false;
        }

        return true;
    }
	
	
	
    private boolean saveToJPEG()
    {
       String v=java.lang.System.getProperty("java.version");
       if (v.indexOf("1.1")==0) 
           return false;
       try 
       {
            // *** This method was added to auto size the images the first time ***
            if (bc.autoSize) 
            {
                bc.setSize(170,90);
                java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                java.awt.Graphics imgTempGraphics = imageTemp.createGraphics();
                bc.paint(imgTempGraphics);
                bc.invalidate();
                imgTempGraphics.dispose();
            }

            // create bufferred image
            java.awt.image.BufferedImage image = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_INT_RGB );
            java.awt.Graphics imgGraphics = image.createGraphics();
            bc.paint(imgGraphics );

            // open file
            java.io.File f=new java.io.File(sFile);
            f.delete();
            java.io.FileOutputStream of=new java.io.FileOutputStream(f);

            // encode buffered image to a jpeg
            //com.sun.image.codec.jpeg.JPEGImageEncoder encoder = com.sun.image.codec.jpeg.JPEGCodec.createJPEGEncoder(of ); BRJ 9/2016 Not compatible with Java 7
            //increase the JPEG quality to 100%
            //com.sun.image.codec.jpeg.JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam( image ); BRJ 9/2016 Not compatible with Java 7
			//param.setQuality(1.0F,true);
            //encoder.setJPEGEncodeParam(param); BRJ 9/2016 Not compatible with java 7. Java 7 now uses ImageIO.write
            // encoder.encode( image,param ); 
		    
			ImageIO.write(image, "jpeg", of);  //BRJ Replaces com.sun.image.codec for Java version 1.4 and up.  9/2016
            of.close();
        
        }
        catch (Exception e) 
        {
            return false;
        }
        return true;
    }
}
