//*****************************************************************
// 
//  JAVA Source for com.idautomation.linear; 2012
//  
//	Copyright IDAutomation.com, Inc. All rights reserved.
//	Redistribution and use of this code in source and/or binary 
//	forms, with or without modification, are permitted provided 
//	that: (1) all copies of the source code retain the above 
//	unmodified copyright notice and this entire unmodified 
//	section of text, (2) You or Your organization owns a valid 
//	Developer License to this product from IDAutomation.com 
//	and, (3) when any portion of this code is bundled in any 
//	form with an application, a valid notice must be provided 
//	within the user documentation, start-up screen or in the 
//	help-about section of the application that specifies 
//	IDAutomation.com as the provider of the Software bundled 
//	with the application.
//*****************************************************************

package com.idautomation.linear.encoder;


import java.awt.Font;
import java.awt.Graphics;
import java.io.File;
import javax.imageio.ImageIO;
//import net.sf.epsgraphics.ColorMode;  ///for Image Generator for Mac
//import net.sf.epsgraphics.EpsGraphics; 


/** This is the class that actually saves the image of the BarCode class. */
public class barCodeEncoder {

   String sFile;
   String sFormat;
   com.idautomation.linear.BarCode bc;
   public boolean result;
/** Constructor used to save an image of the current state of the BarCode object.
@param c BarCode object.
@param psFormat format to save the image as a "GIF" or "JPEG"
@param psFile Name of the saved image file, "Test.gif"
@see encodeImage(com.idautomation.linear.BarCode c,String psFormat,String psFile)*/
  public barCodeEncoder(com.idautomation.linear.BarCode c,String psFormat,String psFile) {

     sFormat=psFormat;
     sFile=psFile;
     bc=c;
     result=encode();
  }
  public barCodeEncoder()
  {
      
  }
/** Has the same functionality as the constructor, can be used to encode images without the need to reinstantiate the barCodeEncoder object.
@param c BarCode object.
@param psFormat format to save the image as a "GIF", "JPEG", "EPS", "PNG"
@param psFile Name of the saved image file, "Test.gif"
@see barCodeEncoder(com.idautomation.linear.BarCode c,String psFormat,String psFile)*/
  public void encodeImage(com.idautomation.linear.BarCode c,String psFormat,String psFile) 
  {
     sFormat=psFormat;
     sFile=psFile;
     bc=c;
     result=encode();
  }
  private boolean encode() 
  {
    if (sFormat.toUpperCase().compareTo("GIF")==0) 
        return saveToGIF();
    if (sFormat.toUpperCase().compareTo("JPEG")==0) 
        return saveToJPEG();
    if (sFormat.toUpperCase().compareTo("EPS")==0) 
        return saveToEPS();
    if (sFormat.toUpperCase().compareTo("PNG")==0) 
        return saveToPNG();
    return false;
  }

   private boolean saveToGIF() 
   {
      String v=java.lang.System.getProperty("java.version");
      if (v.indexOf("1.1")==0) 
          return false;
       try 
       {
            // *** This method was added to auto size the images the first time ***
            if (bc.autoSize) 
            {
                //TB, 10/2/2009, fix bug #363, must set a temporary size, otherwise a null image is created.
                bc.setSize(170,90);
                java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( 100,100,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                //java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                java.awt.Graphics imgTempGraphics = imageTemp.createGraphics();
                bc.paint(imgTempGraphics);
                bc.invalidate();
                imgTempGraphics.dispose();
            }

            //create bufferred image
            java.awt.image.BufferedImage image = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
            //java.awt.Image image=c.createImage(c.getSize().width,c.getSize().height);
            java.awt.Graphics imgGraphics = image.createGraphics();

            bc.paint(imgGraphics);

            // open file
            java.io.File f=new java.io.File(sFile);
            f.delete();
            java.io.FileOutputStream of=new java.io.FileOutputStream(f);

            // encode buffered image to a gif
            GifEncoder encoder = new GifEncoder(image,of);
            encoder.encode();
            of.close();
        }
        catch (Exception e) 
        {
            return false;
        }

        return true;
    }

    private boolean saveToJPEG()
    {
       String v=java.lang.System.getProperty("java.version");
       if (v.indexOf("1.1")==0) 
           return false;
       try 
       {
            // *** This method was added to auto size the images the first time ***
            if (bc.autoSize) 
            {
                bc.setSize(170,90);
                java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                java.awt.Graphics imgTempGraphics = imageTemp.createGraphics();
                bc.paint(imgTempGraphics);
                bc.invalidate();
                imgTempGraphics.dispose();
            }

            // create bufferred image
            java.awt.image.BufferedImage image = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_INT_RGB );
            java.awt.Graphics imgGraphics = image.createGraphics();
            bc.paint(imgGraphics );

            // open file
            java.io.File f=new java.io.File(sFile);
            f.delete();
            java.io.FileOutputStream of=new java.io.FileOutputStream(f);

            // encode buffered image to a jpeg
            //com.sun.image.codec.jpeg.JPEGImageEncoder encoder = com.sun.image.codec.jpeg.JPEGCodec.createJPEGEncoder(of ); BRJ 9/2016 Not compatible with Java 7
            //increase the JPEG quality to 100%
            //com.sun.image.codec.jpeg.JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam( image ); BRJ 9/2016 Not compatible with Java 7
			//param.setQuality(1.0F,true);
            //encoder.setJPEGEncodeParam(param); BRJ 9/2016 Not compatible with java 7. Java 7 now uses ImageIO.write
            // encoder.encode( image,param ); 
		    
			ImageIO.write(image, "jpeg", of);  //BRJ Replaces com.sun.image.codec for Java version 1.4 and up.  9/2016
            of.close();
        }
        catch (Exception e) 
        {
            return false;
        }
        return true;
    }
    private boolean saveToEPS()
    {
        String v=java.lang.System.getProperty("java.version");
        if (v.indexOf("1.1")==0) 
           return false;
        try 
        {
            // *** This method was added to auto size the images the first time ***
            if (bc.autoSize) 
            {
                bc.setSize(170,90);
                java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                java.awt.Graphics imgTempGraphics = imageTemp.createGraphics();
                bc.paint(imgTempGraphics);
                bc.invalidate();
                imgTempGraphics.dispose();
            }
            
            // create bufferred image
            java.awt.image.BufferedImage image = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_INT_RGB );
            
            // open file
            java.io.File f=new java.io.File(sFile);
            f.delete();
            java.io.FileOutputStream of=new java.io.FileOutputStream(f);
            //EpsGraphics eps = new EpsGraphics("test",of,0,0,image.getWidth(),image.getHeight(),ColorMode.COLOR_RGB);
           //java.awt.Graphics epsGraphics = eps.create();
            //epsGraphics.setColor(bc.getBackground());
            //bc.paint(epsGraphics);
            //eps.close();
            of.close();
        }
        catch (Exception e) 
        {
            return false;
        }
        return true;
    }
    private boolean saveToPNG()
    {
       String v=java.lang.System.getProperty("java.version");
       if (v.indexOf("1.1")==0) 
           return false;
       try 
       {
            // *** This method was added to auto size the images the first time ***
            if (bc.autoSize) 
            {
                bc.setSize(170,90);
                java.awt.image.BufferedImage imageTemp = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_BYTE_INDEXED );
                java.awt.Graphics imgTempGraphics = imageTemp.createGraphics();
                bc.paint(imgTempGraphics);
                bc.invalidate();
                imgTempGraphics.dispose();
            }

            // create bufferred image
            java.awt.image.BufferedImage image = new java.awt.image.BufferedImage( bc.getSize().width,bc.getSize().height,java.awt.image.BufferedImage.TYPE_INT_RGB );
            java.awt.Graphics imgGraphics = image.createGraphics();
            bc.paint(imgGraphics );

            // open file
            File f=new File(sFile);
            f.delete();
            // Save as PNG
            ImageIO.write(image, "png", f);
        }
        catch (Exception e) 
        {
            return false;
        }
        return true;
    }
}
